'use strict';

/* eslint-disable */
function forEach(arr, callback) {
  for (var i = 0; arr && i < arr.length; i++) {
    callback && callback(arr[i], i);
  }
}
function objAssign(target, sources) {
  var result = deepClone(target);
  var args = [].slice.call(arguments, 1);

  for (var index = 0; index < args.length; index++) {
    var source = args[index];
    if (source == null) continue;

    for (var key in source) {
      if (Object.prototype.hasOwnProperty.call(source, key)) {
        result[key] = source[key];
      }
    }
  }

  return result;
}
function objKeys(obj) {
  var resutl = [];
  if (!obj) return resutl;

  for (var key in obj) {
    if (Object.prototype.hasOwnProperty.call(obj, key)) {
      resutl.push(key);
    }
  }

  return resutl;
} // 深拷贝：序列化反序列化法 （只能深拷贝对象和数组）

function deepClone(obj) {
  return JSON.parse(JSON.stringify(obj));
}

var ConfigMap = {};

function setConfig(key, value) {
  if (!key) return;
  ConfigMap[key] = value;
}

function getConfig(key, defaultValue) {
  return ConfigMap[key] || defaultValue;
}

function _typeof(obj) {
  "@babel/helpers - typeof";

  if (typeof Symbol === "function" && typeof Symbol.iterator === "symbol") {
    _typeof = function (obj) {
      return typeof obj;
    };
  } else {
    _typeof = function (obj) {
      return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj;
    };
  }

  return _typeof(obj);
}

function aeLog() {

  var msg = '';

  for (var i = 0; arguments && i < arguments.length; i++) {
    if (_typeof(arguments[i]) === 'object') {
      try {
        msg += " ".concat(JSON.stringify(arguments[i]));
      } catch (e) {
        // do nothing
        if (window) {
          console.log('aeLog 异常');
        } else {
          $.writeln('aeLog 异常');
        }
      }
    } else {
      msg += " ".concat(arguments[i]);
    }
  }

  if (window) {
    console.log(msg);
  } else {
    $.writeln(msg);
  }
}

var transformPropertyMap = {
  translateX: {
    "default": '0',
    order: 1,
    name: 'translateX',
    prefix: '(',
    suffix: ')',
    unit: 'px'
  },
  translateY: {
    "default": '0',
    order: 2,
    name: 'translateY',
    prefix: '(',
    suffix: ')',
    unit: 'px'
  },
  scaleX: {
    "default": '1',
    order: 3,
    name: 'scaleX',
    prefix: '(',
    suffix: ')',
    unit: ''
  },
  scaleY: {
    "default": '1',
    order: 4,
    name: 'scaleY',
    prefix: '(',
    suffix: ')',
    unit: ''
  },
  rotate: {
    "default": '0 0',
    order: 5,
    name: 'rotate',
    prefix: '(',
    suffix: ')',
    unit: 'deg'
  },
  rotateX: {
    "default": '0',
    order: 6,
    name: 'rotateX',
    prefix: '(',
    suffix: ')',
    unit: 'deg'
  },
  rotateY: {
    "default": '0',
    order: 7,
    name: 'rotateY',
    prefix: '(',
    suffix: ')',
    unit: 'deg'
  }
};
var normalPropertyMap = {
  width: {
    "default": '0',
    order: -1,
    name: 'width',
    prefix: ':',
    suffix: ';',
    unit: 'px'
  },
  height: {
    "default": '0',
    order: -2,
    name: 'height',
    prefix: ':',
    suffix: ';',
    unit: 'px'
  },
  left: {
    "default": '0',
    order: -3,
    name: 'left',
    prefix: ':',
    suffix: ';',
    unit: 'px'
  },
  top: {
    "default": '0',
    order: -4,
    name: 'top',
    prefix: ':',
    suffix: ';',
    unit: 'px'
  },
  opacity: {
    "default": '1',
    order: -5,
    name: 'opacity',
    prefix: ':',
    suffix: ';',
    unit: ''
  },
  layerTransformOrigin: {
    "default": '50% 50%',
    order: -6,
    name: 'transform-origin',
    prefix: ':',
    suffix: ';',
    unit: 'px'
  },
  vectorTransformOrigin: {
    "default": '50% 50%',
    order: -7,
    name: 'transform-origin',
    prefix: ':',
    suffix: ';',
    unit: '%'
  },
  perspective: {
    "default": 'none',
    order: -8,
    name: 'perspective',
    prefix: ':',
    suffix: ';',
    unit: 'px'
  },
  easing: {
    "default": null,
    order: -9,
    name: 'animation-timing-function',
    prefix: ':',
    suffix: ';',
    unit: ''
  },
  background: {
    "default": null,
    order: -10,
    name: 'background',
    prefix: ':',
    suffix: ';',
    unit: ''
  }
};

function scaleNumber(value, unit) {
  if (!isNumber(value)) {
    return value;
  }

  var scaleRatio = getConfig('IFRAME_SCALE_RATIO', 1);
  return numberShaking(value * scaleRatio, 1);
}

function getArrayStringProperty(value, typeObj) {
  var tempValue = '';
  var arr = [value];

  if (typeof value === 'string') {
    arr = value.split(' ');
  }

  if (arr.length > 1) {
    forEach(arr, function (child) {
      tempValue += "".concat(scaleNumber(child, typeObj.unit)).concat(Number(child) ? typeObj.unit : '', " ");
    });
  } else {
    tempValue = "".concat(scaleNumber(arr[0], typeObj.unit)).concat(Number(arr[0]) ? typeObj.unit : '');
  }

  return tempValue;
}

function mergeTransformProperty(str) {
  if (!str) return '';
  return str.replace(/translateX\((.*?)\) translateY\((.*?)\)/, function (w, w1, w2) {
    return "translate(".concat(w1, ", ").concat(w2, ")");
  }).replace(/scaleX\((.*?)\) scaleY\((.*?)\)/, function (w, w1, w2) {
    if (Number(w1) === Number(w2)) {
      return "scale(".concat(w1, ")");
    } else {
      return "scale(".concat(w1, ", ").concat(w2, ")");
    }
  }).replace(/rotateX\((.*?)\) rotateY\((.*?)\)/, function (w, w1, w2) {
    if (Number(w1) === Number(w2)) {
      return "rotate(".concat(w1, ")");
    } else {
      return "rotate(".concat(w1, ", ").concat(w2, ")");
    }
  });
}

function getPropertyCss(obj, isLastKeyframe) {
  var transformPropertyCss = '';
  var normalPropertyCss = ''; // 对key 进行排序... 为了保证rotate是最后之类的...

  var keyList = objKeys(obj);
  keyList.sort(function (before, after) {
    var keyOrder = function keyOrder(key) {
      if (transformPropertyMap[key]) {
        return transformPropertyMap[key].order;
      } else if (normalPropertyMap[key]) {
        return normalPropertyMap[key].order;
      } else {
        return 0;
      }
    };

    return keyOrder(before) - keyOrder(after);
  });
  aeLog('keyList', keyList);
  forEach(keyList, function (key, index) {
    if (transformPropertyMap[key] && (isLastKeyframe || transformPropertyMap[key]["default"] !== obj[key])) {
      // 当前帧不是最后一帧（最后一帧一定要显示效果） || 当前帧的transform属性不是默认值
      if (!transformPropertyCss) {
        transformPropertyCss += 'transform: ';
      }

      transformPropertyCss += "".concat(transformPropertyMap[key].name).concat(transformPropertyMap[key].prefix).concat(getArrayStringProperty(obj[key], transformPropertyMap[key])).concat(transformPropertyMap[key].suffix, " ");
    }

    if (normalPropertyMap[key] && obj[key] !== undefined) {
      if (index > 0) {
        normalPropertyCss += '\n';
      }

      if (key === 'transformOrigin') {
        normalPropertyCss += "".concat(normalPropertyMap[key].name).concat(normalPropertyMap[key].prefix, " ").concat(getArrayStringProperty(obj[key], normalPropertyMap[key])).concat(normalPropertyMap[key].suffix);
      } else {
        normalPropertyCss += "".concat(normalPropertyMap[key].name).concat(normalPropertyMap[key].prefix, " ").concat(getArrayStringProperty(obj[key], normalPropertyMap[key])).concat(normalPropertyMap[key].suffix);
      }
    }
  }); // transformPropertyCss 正则表达式处理

  transformPropertyCss = mergeTransformProperty(transformPropertyCss);
  return {
    transformPropertyCss: transformPropertyCss,
    normalPropertyCss: normalPropertyCss
  };
}
function isNumber(value) {
  return typeof Number(value) === 'number' && !isNaN(value);
}
function numberShaking(num) {
  var digitCount = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : 1;

  if (!isNumber(num)) {
    return num;
  }

  return Number(Number(num).toFixed(digitCount));
}

function getCssKeyFrameTemplate(item, isLastKeyframe) {
  var _getPropertyCss = getPropertyCss(item, isLastKeyframe),
      transformPropertyCss = _getPropertyCss.transformPropertyCss,
      normalPropertyCss = _getPropertyCss.normalPropertyCss;

  if (transformPropertyCss) {
    transformPropertyCss += ';';
    return "  ".concat(numberShaking(item.offset * 100), "% {\n     ").concat(normalPropertyCss, "\n      ").concat(transformPropertyCss, "\n  }\n\n");
  } else {
    return "  ".concat(numberShaking(item.offset * 100), "% {\n     ").concat(normalPropertyCss, "\n  }\n\n");
  }
} // function syncToNextFrames(animArr) {
//   aeLog('syncToNextFrames');
//   for (let i = 0; i < animArr.length; i++) {
//     let startIndex = i;
//     let arr0 = animArr[startIndex];
//     for (let j = startIndex + 1; j < animArr.length; j++) {
//       arr0 = objAssign(animArr[j], arr0);
//     }
//     animArr[startIndex] = arr0;
//   }
// }
// 第一帧是动画运动的开始，需要拥有后面帧的 所有transform属性


function fillFirstFrame(animArr) {
  var startIndex = 0;
  var arr0 = animArr[startIndex];

  for (var j = startIndex + 1; j < animArr.length; j++) {
    arr0 = objAssign(animArr[j], arr0);
  }

  animArr[startIndex] = arr0;
} // 填充非第一帧


function fillNotFirstFrame(animArr) {
  var _loop = function _loop(i) {
    var temp = animArr[i];
    var normalKeyList = ['left', 'right', 'top', 'bottom'];
    var transformKeyList = ['rotate', 'rotateX', 'rotateY', 'scale', 'scaleX', 'scaleY', 'translate', 'translateX', 'translateY', 'transformOrigin'];
    var mixKeyList = [].concat(normalKeyList, transformKeyList);
    forEach(mixKeyList, function (key) {
      if (temp[key]) {
        hasNoTransformInBehind(animArr, key, i);
      }
    });
  };

  for (var i = 0; i < animArr.length; i++) {
    _loop(i);
  }
} // 首尾添加offset 为0和1


function insertBeginAndEnding(animArr) {
  if (animArr && animArr[0].offset !== 0) {
    animArr.unshift(objAssign(animArr[0], {
      offset: 0
    }));
  }

  if (animArr && animArr[animArr.length - 1].offset !== 1) {
    animArr.push(objAssign(animArr[animArr.length - 1], {
      offset: 1
    }));
  }
} // 如果某一帧拥有某个transform属性，其后面的帧都没有该属性，需要将该帧赋值给后面的帧


function hasNoTransformInBehind(animArr, key, startIndex) {
  var hasNoTransfrom = true;

  for (var i = startIndex + 1; i < animArr.length; i++) {
    var temp = animArr[i];

    if (temp[key]) {
      hasNoTransfrom = false;
      break;
    }
  }

  if (hasNoTransfrom) {
    for (var _i = startIndex + 1; _i < animArr.length; _i++) {
      animArr[_i][key] = animArr[startIndex][key];
    }
  }

  return hasNoTransfrom;
}

function transformToCssAnimate(animateArr, id) {
  var propertyNum = animateArr.length; // 多个数组排序规则:
  // step1. 先把多组的合并成一个 mixArr
  // aeLog('处理前 animateArr', animateArr);

  var mixArr = [];
  forEach(animateArr, function (item) {
    if (item && item.value && item.value.length > 0) {
      forEach(item.value, function (p) {
        mixArr.push(p);
      });
    }
  }); // step2. 将mixArr按照offset排序

  mixArr.sort(function (before, after) {
    return before.offset - after.offset;
  });
  var resultAnimArr = []; // step3. 将mixArr中的 offset item进行合并: 相同的offset合并在一块，合并后的item中可能包含多个 transform property

  forEach(mixArr, function (item) {
    if (resultAnimArr.length > 0 && item.offset === resultAnimArr[resultAnimArr.length - 1].offset) {
      resultAnimArr[resultAnimArr.length - 1] = objAssign(resultAnimArr[resultAnimArr.length - 1], item);
    } else {
      resultAnimArr.push(item);
    }
  }); // aeLog('处理好的resultAnimArr ', resultAnimArr);
  // step4. 填充第一帧

  fillFirstFrame(resultAnimArr); // aeLog('填充完第一帧 ', resultAnimArr);
  // step5. 填充第一帧后面的所有帧

  fillNotFirstFrame(resultAnimArr); // aeLog('填充完最后一帧 ', resultAnimArr);
  // step6. 补上 0 和 1 这两个offset的关键帧

  insertBeginAndEnding(resultAnimArr); // aeLog('补上0和1两个关键帧 ', resultAnimArr);
  // step7. 生成 keyframe css动画

  var result = '';
  forEach(resultAnimArr, function (item, index) {
    result += getCssKeyFrameTemplate(item, index === resultAnimArr.length - 1);
  }); // aeLog('得到的 result', result);

  return result;
}

function getAllKeyframeCss(tree) {
  var id = tree.id,
      name = tree.name,
      _tree$props = tree.props,
      _tree$animate = tree.animate,
      animate = _tree$animate === void 0 ? [] : _tree$animate,
      _tree$children = tree.children,
      children = _tree$children === void 0 ? [] : _tree$children; // aeLog('animate', animate);

  var keyframeCssCode = ''; // 构建当前元素

  if (animate.length > 0) {
    keyframeCssCode += "@keyframes anim-".concat(id, " {\n   ").concat(transformToCssAnimate(animate), "\n}"); // aeLog(keyframeCssCode);
  } // 递归遍历 cssTree


  forEach(children, function (child) {
    keyframeCssCode += '\n' + getAllKeyframeCss(child);
  });
  return keyframeCssCode;
}

function getDomCss(node, keyframeCss) {
  var id = node.id,
      tagName = node.tagName,
      _node$props = node.props,
      props = _node$props === void 0 ? {} : _node$props;
  var defaultPropertyCss = 'position: absolute;';

  var _getPropertyCss = getPropertyCss(props),
      transformPropertyCss = _getPropertyCss.transformPropertyCss,
      normalPropertyCss = _getPropertyCss.normalPropertyCss;

  if (transformPropertyCss) {
    transformPropertyCss += ';';
  }

  if (keyframeCss.indexOf("anim-".concat(id)) > -1) {
    // 说明包含 动画属性
    defaultPropertyCss += "\nanimation: ".concat("anim-".concat(id), " ", numberShaking(props.animDuration, 0), "ms forwards infinite;");
  }

  return "#".concat(id, " {\n  ").concat(defaultPropertyCss, "\n  ").concat(normalPropertyCss, "\n  ").concat(transformPropertyCss, "}\n");
}

function isValidDomSize(size) {
  return isNaN(Number(size)) || Number(size) > 0;
}

function getDomWidthAndHeight(node) {
  var id = node.id,
      _node$props = node.props;
  _node$props = _node$props === void 0 ? {} : _node$props;
  var width = _node$props.width,
      height = _node$props.height,
      _node$children = node.children,
      children = _node$children === void 0 ? [] : _node$children;
  var containerWidth = 'auto';
  var containerHeight = 'auto';
  var containerWH_Ratio = 1; // 构建当前元素

  if (isValidDomSize(width) && isValidDomSize(height) && children.length > 0) {
    var child = children[0];
    containerWidth = child.props && child.props.width;
    containerHeight = child.props && child.props.height;
    containerWH_Ratio = (containerWidth / containerHeight).toFixed(2);
  }

  return {
    containerWidth: containerWidth,
    containerHeight: containerHeight,
    containerWH_Ratio: containerWH_Ratio
  };
}

function getDomHtml(node, keyframeCss) {
  var id = node.id,
      _node$props2 = node.props;
  _node$props2 = _node$props2 === void 0 ? {} : _node$props2;
  var width = _node$props2.width,
      height = _node$props2.height,
      _node$children2 = node.children,
      children = _node$children2 === void 0 ? [] : _node$children2;
  var domHtml = '';
  var domCss = '';
  var containerWidth = 'auto';
  var containerHeight = 'auto';
  var containerWH_Ratio = 1; // 构建当前元素

  if (isValidDomSize(width) && isValidDomSize(height)) {
    if (children.length > 0) {
      var childrenDivHtml = '';
      forEach(children, function (child, index) {
        var _getDomHtml = getDomHtml(child, keyframeCss),
            html = _getDomHtml.domHtml,
            css = _getDomHtml.domCss;

        childrenDivHtml += html;
        domCss += '\n' + css;

        if (index === 0) {
          containerWidth = child.props && child.props.width;
          containerHeight = child.props && child.props.height;
          containerWH_Ratio = (containerWidth / containerHeight).toFixed(2);
        }
      });
      domHtml = "<div id=\"".concat(id, "\">").concat(childrenDivHtml, "</div>");
    } else {
      domHtml = "<div id=\"".concat(id, "\"></div>");
    }

    domCss += '\n' + getDomCss(node, keyframeCss);
  } // aeLog('domHtml', domHtml);
  // aeLog('domCss', domCss);


  return {
    domHtml: domHtml,
    domCss: domCss,
    containerWidth: containerWidth,
    containerHeight: containerHeight,
    containerWH_Ratio: containerWH_Ratio
  };
}

function htmlRender(cssTree) {
  // aeLog('traverseNode', traverseNode);
  // 先收集所有的动效keyframes
  var keyframeCss = getAllKeyframeCss(cssTree);

  var _getDomHtml = getDomHtml(cssTree, keyframeCss),
      domHtml = _getDomHtml.domHtml,
      domCss = _getDomHtml.domCss; // console.log('domHtml', domHtml);
  // console.log('domCss', domCss);


  var htmlTemplate = '<!DOCTYPE html>\n' + '<html lang="en">\n' + '<head>\n' + '    <meta charset="UTF-8">\n' + '    <style type="text/css">\n' + "body{margin: 0;position: fixed;top: 0;right: 0;bottom: 0;left: 0;display: flex;align-items: center;justify-content: center;background: #232322}" + "".concat(domCss, "\n") + "".concat(keyframeCss, "\n") + '    </style>\n' + '</head>\n' + '<body>\n' + "".concat(domHtml, "\n") + '</body>\n' + '</html>\n';
  return {
    htmlTemplate: htmlTemplate,
    domHtml: domHtml,
    domCss: domCss,
    keyframeCss: keyframeCss
  };
}

exports.getDomWidthAndHeight = getDomWidthAndHeight;
exports.htmlRender = htmlRender;
exports.setConfig = setConfig;
