import {getDomCss} from "./domCss";
import {forEach} from "../polyfill";

function isValidDomSize(size) {
  return isNaN(Number(size)) || Number(size) > 0;
}

function getDomWidthAndHeight(node) {
  const {
    id,
    props: {
      width,
      height,
    } = {},
    children = [],
  } = node;
  let containerWidth = 'auto';
  let containerHeight = 'auto';
  let containerWH_Ratio = 1;
  // 构建当前元素
  if (isValidDomSize(width) && isValidDomSize(height) && children.length > 0) {
    const child = children[0];
    containerWidth = child.props && child.props.width;
    containerHeight = child.props && child.props.height;
    containerWH_Ratio = (containerWidth / containerHeight).toFixed(2);
  }
  return {containerWidth, containerHeight, containerWH_Ratio};
}


function getDomHtml(node, keyframeCss) {
  const {
    id,
    props: {
      width,
      height,
    } = {},
    children = [],
  } = node;
  let domHtml = '';
  let domCss = '';
  let containerWidth = 'auto';
  let containerHeight = 'auto';
  let containerWH_Ratio = 1;
  // 构建当前元素
  if (isValidDomSize(width) && isValidDomSize(height)) {
    if (children.length > 0) {
      let childrenDivHtml = '';
      forEach(children, (child, index) => {
        const {domHtml: html, domCss: css} = getDomHtml(child, keyframeCss);
        childrenDivHtml += html;
        domCss += '\n' + css;
        if (index === 0) {
          containerWidth = child.props && child.props.width;
          containerHeight = child.props && child.props.height;
          containerWH_Ratio = (containerWidth / containerHeight).toFixed(2);
        }
      });
      domHtml = `<div id="${id}">${childrenDivHtml}</div>`;
    } else {
      domHtml = `<div id="${id}"></div>`;
    }
    domCss += '\n' + getDomCss(node, keyframeCss);
  }
  // aeLog('domHtml', domHtml);
  // aeLog('domCss', domCss);
  return {domHtml, domCss, containerWidth, containerHeight, containerWH_Ratio};
}

export {
  getDomWidthAndHeight,
  getDomHtml,
};
