import {getPropertyCss, numberShaking} from "../util";
import {forEach, objAssign} from "../polyfill";
import {aeLog} from "../log";

function getCssKeyFrameTemplate(item, isLastKeyframe) {
  let {transformPropertyCss, normalPropertyCss} = getPropertyCss(item, isLastKeyframe);
  if (transformPropertyCss) {
    transformPropertyCss += ';';
    return `  ${numberShaking(item.offset * 100)}% {\n     ${normalPropertyCss}\n      ${transformPropertyCss}\n  }\n\n`;
  } else {
    return `  ${numberShaking(item.offset * 100)}% {\n     ${normalPropertyCss}\n  }\n\n`;
  }
}

// function syncToNextFrames(animArr) {
//   aeLog('syncToNextFrames');
//   for (let i = 0; i < animArr.length; i++) {
//     let startIndex = i;
//     let arr0 = animArr[startIndex];
//     for (let j = startIndex + 1; j < animArr.length; j++) {
//       arr0 = objAssign(animArr[j], arr0);
//     }
//     animArr[startIndex] = arr0;
//   }
// }

// 第一帧是动画运动的开始，需要拥有后面帧的 所有transform属性
function fillFirstFrame(animArr) {
  let startIndex = 0;
  let arr0 = animArr[startIndex];
  for (let j = startIndex + 1; j < animArr.length; j++) {
    arr0 = objAssign(animArr[j], arr0);
  }
  animArr[startIndex] = arr0;
}

// 填充非第一帧
function fillNotFirstFrame(animArr) {
  for (let i = 0; i < animArr.length; i++) {
    let temp = animArr[i];
    const normalKeyList = ['left', 'right', 'top', 'bottom'];
    const transformKeyList = ['rotate', 'rotateX', 'rotateY', 'scale', 'scaleX', 'scaleY', 'translate', 'translateX', 'translateY', 'transformOrigin'];
    const mixKeyList = [...normalKeyList, ...transformKeyList];
    forEach(mixKeyList, (key) => {
      if (temp[key]) {
        hasNoTransformInBehind(animArr, key, i);
      }
    });
  }
}

// 首尾添加offset 为0和1
function insertBeginAndEnding(animArr) {
  if (animArr && animArr[0].offset !== 0) {
    animArr.unshift(objAssign(animArr[0], {offset: 0}));
  }
  if (animArr && animArr[animArr.length - 1].offset !== 1) {
    animArr.push(objAssign(animArr[animArr.length - 1], {offset: 1}));
  }
}

// 如果某一帧拥有某个transform属性，其后面的帧都没有该属性，需要将该帧赋值给后面的帧
function hasNoTransformInBehind(animArr, key, startIndex) {
  let hasNoTransfrom = true;
  for (let i = startIndex + 1; i < animArr.length; i++) {
    let temp = animArr[i];
    if (temp[key]) {
      hasNoTransfrom = false;
      break;
    }
  }
  if (hasNoTransfrom) {
    for (let i = startIndex + 1; i < animArr.length; i++) {
      animArr[i][key] = animArr[startIndex][key];
    }
  }
  return hasNoTransfrom;
}

function transformToCssAnimate(animateArr, id) {
  const propertyNum = animateArr.length;
  // 多个数组排序规则:
  // step1. 先把多组的合并成一个 mixArr
  // aeLog('处理前 animateArr', animateArr);
  const mixArr = [];
  forEach(animateArr, (item) => {
    if (item && item.value && item.value.length > 0) {
      forEach(item.value, p => {
        mixArr.push(p);
      });
    }
  });
  // step2. 将mixArr按照offset排序
  mixArr.sort((before, after) => before.offset - after.offset);
  const resultAnimArr = [];
  // step3. 将mixArr中的 offset item进行合并: 相同的offset合并在一块，合并后的item中可能包含多个 transform property
  forEach((mixArr), (item) => {
    if (resultAnimArr.length > 0 && item.offset === resultAnimArr[resultAnimArr.length - 1].offset) {
      resultAnimArr[resultAnimArr.length - 1] = objAssign(resultAnimArr[resultAnimArr.length - 1], item);
    } else {
      resultAnimArr.push(item);
    }
  });
  // aeLog('处理好的resultAnimArr ', resultAnimArr);
  // step4. 填充第一帧
  fillFirstFrame(resultAnimArr);
  // aeLog('填充完第一帧 ', resultAnimArr);
  // step5. 填充第一帧后面的所有帧
  fillNotFirstFrame(resultAnimArr);
  // aeLog('填充完最后一帧 ', resultAnimArr);
  // step6. 补上 0 和 1 这两个offset的关键帧
  insertBeginAndEnding(resultAnimArr);
  // aeLog('补上0和1两个关键帧 ', resultAnimArr);
  // step7. 生成 keyframe css动画
  let result = '';
  forEach(resultAnimArr, (item, index) => {
    result += getCssKeyFrameTemplate(item, index === resultAnimArr.length - 1);
  });
  // aeLog('得到的 result', result);
  return result;
}

export function getAllKeyframeCss(tree) {
  const {
    id,
    name,
    props = {},
    animate = [],
    children = [],
  } = tree;
  // aeLog('animate', animate);
  let keyframeCssCode = '';
  // 构建当前元素
  if (animate.length > 0) {
    keyframeCssCode += `@keyframes anim-${id} {\n   ${transformToCssAnimate(animate, id)}\n}`;
    // aeLog(keyframeCssCode);
  }
  // 递归遍历 cssTree
  forEach(children, (child) => {
    keyframeCssCode += '\n' + getAllKeyframeCss(child);
  });
  return keyframeCssCode;
}
